package main

import (
	"bytes"
	"encoding/json"
	"encoding/xml"
    "fmt"
	"io/ioutil"
	"net/http"

	"github.com/JalalJaberi/gosoap"
)

const (
	TYPE_SOAP = 1
	TYPE_REST = 2
)

type UserCreditResponse struct {
	XMLName	xml.Name	`xml:"return"`
	Status		int64	`xml:"status"`
	Value		int64	`xml:"value"`
	Description	string	`xml:"description"`	
}

type GetUserCreditResponse struct {
	XMLName         	xml.Name			`xml:"GetUserCreditResponse"`
	UserCreditResponse	UserCreditResponse	`xml:"return"`
}

type IdehPayamClientApi struct {
	ServiceType		int		`default:1`
	Username		string	`default:""`
	Password		string	`default:""`
	_lastError		string	`default:""`
	_lastResponse	string	`default:""`
}

func (client IdehPayamClientApi) GetRestUrl() string {
	return "http://localhost:8800/idehpayam/new-api/rest.php"
}

func (client IdehPayamClientApi) GetSoapUrl() string {
	return "http://localhost:8800/idehpayam/new-api/soap.php?wsdl"
}

func (client IdehPayamClientApi) GetLastError() string {
	return client._lastError
}

func (client IdehPayamClientApi) GetLastResponse() string {
		return client._lastResponse
}

func (client IdehPayamClientApi) GetUserCredit() int {
	if (client.ServiceType == TYPE_REST) {
		var jsonStr = []byte(`{"operation": "GetUserCredit", "username":"` + client.Username + `", "password":"` + client.Password + `"}`)
		httpRequsest, httpErr := http.NewRequest("POST", client.GetRestUrl(), bytes.NewBuffer(jsonStr))
	
		if httpErr != nil {
			client._lastError = httpErr.Error()
			return 0
		}
	
		httpRequsest.Header.Set("Content-Type", "application/json")
		httpClient := &http.Client{}
		response, clientErr := httpClient.Do(httpRequsest)
	
		if clientErr != nil {
			client._lastError = clientErr.Error()
			return 0
		}
	
		defer response.Body.Close()
	
		// fmt.Println("Response status:", response.Status)
	
		if response.Status != "200 OK" {
			client._lastError = "Response status is not OK"
			return 0
		}
	
		body, bodyError := ioutil.ReadAll(response.Body)
	
		if bodyError != nil {
			client._lastError = bodyError.Error()
			return 0
		}
	
		// fmt.Println("response Body:", string(body))
		client._lastResponse = string(body)
	
		var data map[string]interface{}
		if parseErr := json.Unmarshal(body, &data); parseErr != nil {
			client._lastError = parseErr.Error()
			return 0
		}
	
		statusField, statusIsSet := data["status"]
		if (!statusIsSet) {
			client._lastError = "Status is not set in response"
			return 0
		}
	
		status := statusField.(float64)
		if status != 200 {
			client._lastError = "Status is not 200"
			return 0
		}
	
		result := data["result"].(map[string]interface{})
		innerStatus := result["status"]
		if innerStatus.(float64) == 0 {
			return int(result["value"].(float64))
		} else {
			client._lastError = "returned status: " + innerStatus.(string)
		}
	
		return 0
	} else if (client.ServiceType == TYPE_SOAP) {
		soap, soapErr := gosoap.SoapClient(client.GetSoapUrl())
		if (soapErr != nil) {
			client._lastError = soapErr.Error()
			return 0
		}
		params := gosoap.Params{
			"username": client.Username,
			"password": client.Password,
		}
		response, callErr := soap.Call("GetUserCredit", params)
		if (callErr != nil) {
			client._lastError = callErr.Error()
			return 0
		}
		client._lastResponse = string(response.Body)
		var getUserCreditResponse GetUserCreditResponse
		parseErr := response.Unmarshal(&getUserCreditResponse)
		if (parseErr != nil) {
			client._lastError = parseErr.Error()
			return 0
		}
		if getUserCreditResponse.UserCreditResponse.Status == 0 {
			return int(getUserCreditResponse.UserCreditResponse.Value)
		} else {
			client._lastError = "Server error code: " + string(getUserCreditResponse.UserCreditResponse.Status)
			return 0;
		}
	}
	return 0;
}

func main() {
	client := IdehPayamClientApi{
		Username:		"prodo",
		Password:		"123456",
		ServiceType:	TYPE_SOAP,
	}
	credit := client.GetUserCredit()
	fmt.Println(credit)
}
